package cz.nic.tablexia;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.utils.reflect.ClassReflection;
import com.badlogic.gdx.utils.reflect.ReflectionException;

import java.util.Locale;

import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.event.LocaleChangedEvent;
import cz.nic.tablexia.game.GameDefinition;
import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.screen.gamemenu.GameMenuScreen;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.Utility;

public class TablexiaSettings {

    private static final int     DEFAULT_SCREEN_WIDTH      = 1000;
    private static final double  MAXIMUM_RATIO             = 9.0 / 16.0;
    private static final int     MIN_SCREEN_HEIGHT         = (int) (DEFAULT_SCREEN_WIDTH * MAXIMUM_RATIO);
    private static final boolean DEBUG_SHOW_BOUNDING_BOXES = true;

    public static final Class<? extends AbstractTablexiaScreen<?>> INITIAL_SCREEN = GameMenuScreen.class;

    private static final String PREFERENCES_KEY    = "cz.nic.tablexia.";
    public static final  String LOCALE_KEY         = "locale";
    public static final  String CURRENT_SCREEN_KEY = "current_screen";

    private static final String IDE_BUILD_VERSION_NAME = "DEVEL";

    private final BuildType BUILD_TYPE;
    private final String    VERSION_NAME;

    private Preferences      preferences;
    private LocaleDefinition systemLocale;
    private LocaleDefinition selectedLocale;


    public static enum BuildType {

        RELEASE("release", false, Log.TablexiaLogLevel.ERROR),
        DEBUG("debug", true, Log.TablexiaLogLevel.DEBUG),
        DEVEL("devel", true, Log.TablexiaLogLevel.DEBUG);

        private final static BuildType FALLBACK_VARIANT = BuildType.DEVEL;

        private final String               key;
        private final boolean              isDebug;
        private final Log.TablexiaLogLevel logLevel;

        private BuildType(String key, boolean isDebug, Log.TablexiaLogLevel logLevel) {
            this.key = key;
            this.isDebug = isDebug;
            this.logLevel = logLevel;
        }

        public String getKey() {
            return key;
        }

        public boolean isDebug() {
            return isDebug;
        }

        public Log.TablexiaLogLevel getLogLevel() {
            return logLevel;
        }

        public static BuildType getBuildTypeForKey(String key) {
            for (BuildType buildType : BuildType.values()) {
                if (buildType.key.equals(key)) {
                    return buildType;
                }
            }
            return FALLBACK_VARIANT;
        }
    }

    public enum LocaleDefinition {

        SYSTEM(null, "system", ApplicationTextManager.LANGUAGE_SYSTEM),
        cs_CZ(new Locale("cs", "CZ"), "cs_CZ", ApplicationTextManager.LANGUAGE_CZECH),
        sk_SK(new Locale("sk", "SK"), "sk_SK", ApplicationTextManager.LANGUAGE_SLOVAK);

        public final static  LocaleDefinition DEFAULT_LOCALE   = LocaleDefinition.cs_CZ;
        private final static LocaleDefinition FALLBACK_VARIANT = LocaleDefinition.SYSTEM;

        private final Locale locale;
        private final String localeKey;
        private final String descriptionKey;

        private LocaleDefinition(Locale locale, String localeKey, String descriptionKey) {
            this.locale = locale;
            this.localeKey = localeKey;
            this.descriptionKey = descriptionKey;
        }

        public String getLocaleKey() {
            return localeKey;
        }

        public Locale getLocale() {
            return locale != null ? locale : instance.systemLocale.locale;
        }

        @Override
        public String toString() {
            if (ApplicationTextManager.getInstance().update()) {
                return ApplicationTextManager.getInstance().getText(descriptionKey);
            } else {
                return name();
            }
        }

        public static LocaleDefinition getLocaleDefinitionForKey(String key) {
            for (LocaleDefinition localeDefinition : LocaleDefinition.values()) {
                if (localeDefinition.localeKey.equals(key)) {
                    return localeDefinition;
                }
            }
            return FALLBACK_VARIANT;
        }

        public static LocaleDefinition getLocaleDefinitionForLocale(Locale locale) {
            for (LocaleDefinition localeDefinition : LocaleDefinition.values()) {
                if (localeDefinition.locale != null && localeDefinition.locale.equals(locale)) {
                    return localeDefinition;
                }
            }
            return DEFAULT_LOCALE;
        }
    }


//////////////////////////// SETTINGS SINGLETON

    private static TablexiaSettings instance;

    private TablexiaSettings(BuildType buildType, Locale systemLocale, String versionName) {
        BUILD_TYPE = buildType;
        VERSION_NAME = versionName == null ? IDE_BUILD_VERSION_NAME : versionName;
        this.systemLocale = LocaleDefinition.getLocaleDefinitionForLocale(systemLocale);
    }

    public static TablexiaSettings getInstance() {
        if (instance == null) {
            String exceptionMessage = "Tablexia settings is not initialized!";
            Log.err(TablexiaSettings.class, exceptionMessage);
            throw new IllegalStateException(exceptionMessage);
        }
        return instance;
    }

    public static void init(boolean debug, Locale systemLocale, String versionName) {
        TablexiaSettings.init(debug ? BuildType.DEBUG : BuildType.RELEASE, systemLocale, versionName);
    }

    public static void init(String buildTypeKey, Locale systemLocale, String versionName) {
        TablexiaSettings.init(BuildType.getBuildTypeForKey(buildTypeKey), systemLocale, versionName);
    }

    public static void init(BuildType buildType, Locale systemLocale, String versionName) {
        if (instance != null) {
            String exceptionMessage = "Tablexia settings already initialized!";
            Log.err(TablexiaSettings.class, exceptionMessage);
            throw new IllegalStateException(exceptionMessage);
        }

        instance = new TablexiaSettings(buildType, systemLocale, versionName);
    }


//////////////////////////// LIBGDX PREFERENCES

    public void loadPreferences(boolean resetState) {
        preferences = Gdx.app.getPreferences(PREFERENCES_KEY + BUILD_TYPE.getKey());
        selectedLocale = LocaleDefinition.getLocaleDefinitionForKey(preferences.getString(LOCALE_KEY));
        if (resetState) {
            preferences.remove(CURRENT_SCREEN_KEY);
            preferences.flush();
        }
    }


//////////////////////////// SETTINGS ACCESS

	public boolean isDebug() {
		return BUILD_TYPE.isDebug();
	}

    public String getVersionName() {
        return VERSION_NAME;
    }

    public boolean isShowBoundingBoxes() {
        return isDebug() && DEBUG_SHOW_BOUNDING_BOXES;
    }

    public Log.TablexiaLogLevel getLogLevel() {
        return BUILD_TYPE.getLogLevel();
    }

    public void setLocale(LocaleDefinition localeDefinition) {
        LocaleDefinition lastLocale = selectedLocale;
        selectedLocale = localeDefinition;
        preferences.putString(LOCALE_KEY, localeDefinition.getLocaleKey());
        preferences.flush();
        if (!lastLocale.getLocale().getLanguage().equals(localeDefinition.getLocale().getLanguage())) {
            ApplicationBus.getInstance().post(new LocaleChangedEvent(localeDefinition)).asynchronously();
        }
    }

    public LocaleDefinition getLocaleDefinition() {
        return selectedLocale;
    }

    public Locale getLocale() {
        return selectedLocale.getLocale();
    }

    public void setCurrentScreen(AbstractTablexiaScreen screen) {
        if (screen.hasState()) {
            preferences.putString(CURRENT_SCREEN_KEY, screen.getClass().getName());
            preferences.flush();
        }
    }

    public AbstractTablexiaScreen<?> getCurrentScreen() {
        String screenClassName = preferences.getString(CURRENT_SCREEN_KEY, null);
        if (screenClassName != null) {
            try {
                return Utility.getScreenForScreenClass(ClassReflection.forName(screenClassName));
            } catch (ReflectionException e) {
                Log.err(getClass(), "Cannot load screen class: " + screenClassName, e);
            }
        }
        return null;
    }

    public void setGameDifficulty(GameDefinition gameDefinition, GameDifficulty gameDifficulty) {
        preferences.putString(getGameDifficultyPreferncesKey(gameDefinition), gameDifficulty.name());
        preferences.flush();
    }

    public GameDifficulty getGameDifficulty(GameDefinition gameDefinition) {
        return GameDifficulty.valueOf(GameDifficulty.class, preferences.getString(getGameDifficultyPreferncesKey(gameDefinition), GameDifficulty.DEFAULT_DIFFICULTY.name()));
    }

    private String getGameDifficultyPreferncesKey(GameDefinition gameDefinition) {
        return "GAME_DIFFICULTY_" + gameDefinition.name();
    }


//////////////////////////// SCREEN SIZE

    public static int getWorldWidth() {
        return DEFAULT_SCREEN_WIDTH;
    }

    public static double getMaximumRatio() {
        return MAXIMUM_RATIO;
    }

    public static int getMinWorldHeight() {
        return MIN_SCREEN_HEIGHT;
    }
}