package cz.nic.tablexia.debug;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.Label.LabelStyle;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Disposable;

import net.engio.mbassy.listener.Handler;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentLinkedQueue;

import cz.nic.tablexia.TablexiaApplication.ScreenChangedEvent;
import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.graphics.ColorTexture;
import cz.nic.tablexia.screen.AbstractTablexiaScreen.ScreenInfoEvent;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.alpha;

/**
 * Debug info container
 *
 * @author Matyáš Latner
 */
public class DebugInfo extends Table implements Disposable {

    private static class DebugInfoComponent extends Table {

        private Map<String, Label> infoLabelMap;
        private LabelStyle labelStyle;

        public DebugInfoComponent() {
            infoLabelMap = new HashMap<String, Label>();
            setBackground(new TextureRegionDrawable(new TextureRegion(new ColorTexture(1, 1, BACKGROUND_COLOR))));
            addAction(alpha(BACKGROUND_ALPHA));
            labelStyle = new LabelStyle(new BitmapFont(), FONT_COLOR);
        }

        public synchronized void setInfoValue(String infoKey, String infoValue) {
            Label label = infoLabelMap.get(infoKey);
            if (label == null) {
                createInfoLabel(infoKey, infoValue);
            } else {
                label.setText(infoKey + KEY_COLON + infoValue);
            }
        }

        private void clean() {
            clearChildren();
            infoLabelMap.clear();
        }

        private void createInfoLabel(String infoKey, String infoValue) {
            Label infoLabel = new Label(infoKey + KEY_COLON + infoValue, labelStyle);
            infoLabelMap.put(infoKey, infoLabel);
            clearChildren();

            for (Label label : infoLabelMap.values()) {
                Cell<Label> cell = add(label);
                if (getCells().size > 1) {
                    cell.pad(0, INFO_PADDING, 0, 0);
                }
            }
        }

        @Override
        public void setSize(float width, float height) {
            super.setSize(width, height);
        }
    }


    private static final String NATIVE_HEAP = "Native Heap";
    private static final String JAVA_HEAP = "Java Heap";
    private static final String FPS = "FPS";
    private static final String LOCALE = "Locale";
    private static final String VERSION = "Version";

    private static final String SCREEN_NAME = "Screen Name";

    private static final int INFO_PADDING = 20;
    private static final int MB_SIZE = 1000000;
    private static final String UNIT_MB = " MB";
    private static final String KEY_COLON = ": ";

    private static final Color FONT_COLOR = Color.WHITE;
    private static final Color BACKGROUND_COLOR = Color.BLACK;
    private static final float BACKGROUND_ALPHA = 0.5f;

    private DebugInfoComponent applicationDebugInfo;
    private DebugInfoComponent screenNameInfo;
    private DebugInfoComponent screenDebugInfo;

    private ConcurrentLinkedQueue<ScreenInfoEvent> screenDebugInfoQueue = new ConcurrentLinkedQueue<ScreenInfoEvent>();
    private ConcurrentLinkedQueue<ScreenChangedEvent> screenNameInfoQueue = new ConcurrentLinkedQueue<ScreenChangedEvent>();

    public DebugInfo() {
        ApplicationBus.getInstance().subscribe(this);

        applicationDebugInfo = new DebugInfoComponent();
        screenNameInfo = new DebugInfoComponent();
        screenDebugInfo = new DebugInfoComponent();

        Table topTable = new Table();
        topTable.add(screenNameInfo).expandX();
        topTable.add(screenDebugInfo).pad(0, INFO_PADDING, 0, 0).expandX();
        add(topTable);
        row();
        add(new Container<Actor>()).expand();
        row();
        add(applicationDebugInfo).expandX();
    }

    public void update() {
        applicationDebugInfo.setInfoValue(FPS, "" + Gdx.graphics.getFramesPerSecond());
        applicationDebugInfo.setInfoValue(JAVA_HEAP, ("" + Gdx.app.getJavaHeap() / MB_SIZE) + UNIT_MB);
        applicationDebugInfo.setInfoValue(NATIVE_HEAP, ("" + Gdx.app.getNativeHeap() / MB_SIZE) + UNIT_MB);
        applicationDebugInfo.setInfoValue(LOCALE, "" + TablexiaSettings.getInstance().getLocale());
        applicationDebugInfo.setInfoValue(VERSION, "" + TablexiaSettings.getInstance().getVersionName());

        while (!screenDebugInfoQueue.isEmpty()) {
            ScreenInfoEvent screenInfoEvent = screenDebugInfoQueue.poll();
            screenDebugInfo.setInfoValue(screenInfoEvent.getInfoKey(), screenInfoEvent.getInfoValue());
        }

        while (!screenNameInfoQueue.isEmpty()) {
            screenNameInfo.setInfoValue(SCREEN_NAME, "" + screenNameInfoQueue.poll().getScreenClass().getSimpleName());
        }
    }

    public void onResize() {
        setBounds(0,getStage().getCamera().position.y - getStage().getHeight() / 2,getStage().getWidth(), getStage().getHeight());
    }

    @Handler
    public void handleScreenChangedEvent(ScreenChangedEvent screenChangedEvent) {
        screenDebugInfo.clean();
        screenNameInfoQueue.add(screenChangedEvent);
    }

    @Handler
    public void handleScreenInfoEvent(ScreenInfoEvent screenInfoEvent) {
        screenDebugInfoQueue.add(screenInfoEvent);
    }

    @Override
    public void dispose() {
        ApplicationBus.getInstance().unsubscribe(this);
    }
}
