package cz.nic.tablexia.game;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Cell;
import com.badlogic.gdx.scenes.scene2d.ui.Dialog;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.ui.Table;

import java.util.List;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.game.common.RandomAccess;
import cz.nic.tablexia.game.difficulty.GameDifficulty;
import cz.nic.tablexia.graphics.ColorTexture;
import cz.nic.tablexia.loader.application.ApplicationTextManager;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.util.Log;
import cz.nic.tablexia.util.ui.dialog.TwoButtonDialog;
import cz.nic.tablexia.util.ui.dialog.text.DialogTextContent;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.alpha;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.delay;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.fadeIn;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.fadeOut;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.run;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.sequence;

public abstract class AbstractTablexiaGame<T> extends AbstractTablexiaScreen<T> {

    public static final String RANDOM_SEED_SCREEN_INFO_LABEL        = "Random Seed";
    public static final String GAME_DIFFICULTY_SCREEN_INFO_LABEL    = "Game Difficulty";
    private static final int   SCREEN_TRANSACTION_FADE_DELAY	    = 1;

    public static final String VICTORYSCREEN_RESULTSBANNER          = "application/victoryscreen/resultsbanner.png";
    public static final String VICTORYSCREEN_RATINGSTAR_DISABLED    = "application/victoryscreen/ratingstar_disabled.png";
    public static final String VICTORYSCREEN_RATINGSTAR_ENABLED     = "application/victoryscreen/ratingstar_enabled.png";

    private RandomAccess    randomAccess;
    private GameDifficulty  gameDifficulty;
    private Actor           screenDimmer;

    public AbstractTablexiaGame() {
        final Texture texture = new ColorTexture(Gdx.graphics.getWidth(), Gdx.graphics.getHeight(), Color.BLACK);
        screenDimmer = new Actor(){

            public void draw(Batch batch, float parentAlpha) {
                batch.setColor(getColor());
                batch.draw(texture, 0, 0);
                batch.setColor(Color.WHITE);
            };
        };
        screenDimmer.addAction(alpha(0));
        getStage().addActor(screenDimmer);
    }


//////////////////////////// SCREEN PAUSE

    public boolean canBePaused() {
        return true;
    }


//////////////////////////// ABSTRACT TABLEXIA SCREEN LIFECYCLE


    @Override
    protected final void prepareScreenTextureAssetNames(List<String> textureFileNames) {
        textureFileNames.add(VICTORYSCREEN_RESULTSBANNER);
        textureFileNames.add(VICTORYSCREEN_RATINGSTAR_DISABLED);
        textureFileNames.add(VICTORYSCREEN_RATINGSTAR_ENABLED);
        prepareGameTextureAssetNames(textureFileNames);
    }

    @Override
    protected final T prepareScreenData() {
        randomAccess = new RandomAccess();
        printScreenInfo(RANDOM_SEED_SCREEN_INFO_LABEL, "" + getRandomAccess().getRandomSeed());

        gameDifficulty = TablexiaSettings.getInstance().getGameDifficulty(GameDefinition.getGameDefinitionForClass((Class<? extends AbstractTablexiaGame<?>>)getClass()));
        printScreenInfo(GAME_DIFFICULTY_SCREEN_INFO_LABEL, "" + gameDifficulty.name());

        Log.info(getClass(), "Start game data loading with random seed: " + getRandomAccess().getRandomSeed() + " and difficulty: " + getGameDifficulty());

        return prepareGameData();
    }

    @Override
    protected final void screenLoaded() {
        performShowTransaction(new Runnable() {
            @Override
            public void run() {
                screenVisible();
            }
        });
        gameLoaded();
    }


//////////////////////////// ABSTRACT TABLEXIA GAME LIFECYCLE

    protected T prepareGameData() {
        return null;
    }

    protected void prepareGameTextureAssetNames(List<String> textureFileNames) {}

    protected void gameLoaded() {};


//////////////////////////// ABSTRACT TABLEXIA GAME

    private final float DIALOG_HEIGHT     = AbstractTablexiaGame.this.getStage().getHeight() * (3f/4);
    private final float DIALOG_WIDTH      = DIALOG_HEIGHT * 1.2f;
    private final float DIALOG_POSITION_X = (AbstractTablexiaGame.this.getStage().getWidth() / 2) - (DIALOG_WIDTH / 2);
    private final float DIALOG_POSITION_Y = (AbstractTablexiaGame.this.getStage().getHeight() / 2) - (DIALOG_HEIGHT / 2);

    private class VictoryDialog extends TwoButtonDialog {

        public static final int   RATINGSTARS_COUNT = 3;
        public static final float RATINGSTAR_DELAY  = 0.75f;

        public static final float RESULTSBANNER_WIDTH_RATIO = 2f / 3;
        public static final float RESULTSBANNER_TOPPADDING_RATIO = 1f / 30;
        public static final float RATINGSTAR_HEIGHT_RATIO = 1f / 8;

        Image[] ratingStarsEnabled;

        public VictoryDialog() {
            super(DIALOG_POSITION_X,
                  DIALOG_POSITION_Y,
                  DIALOG_WIDTH,
                  DIALOG_HEIGHT,
                  BackGroundType.DIALOG_SQUARE_BORDERLINES,
                  new DialogTextContent("TEXT"),
                  ButtonType.BLUE,
                  ButtonType.GREEN,
                  ApplicationTextManager.getInstance().getText(ApplicationTextManager.ApplicationTextsAssets.VICTORYSCREEN_BUTTON_CHANGEGAME),
                  ApplicationTextManager.getInstance().getText(ApplicationTextManager.ApplicationTextsAssets.VICTORYSCREEN_BUTTON_REPLAY));
        }

        @Override
        protected void prepareContent() {
            // RESULTS BANNER
            Image resultsBanner = new Image(getTexture(VICTORYSCREEN_RESULTSBANNER));
            float resultBannerWidth = getWidth() * RESULTSBANNER_WIDTH_RATIO;
            float resultsBannerSizeRatio = resultBannerWidth / resultsBanner.getWidth();
            getContentTable().add(resultsBanner).width(resultBannerWidth).height(resultsBanner.getHeight() * resultsBannerSizeRatio).padTop(getHeight() * RESULTSBANNER_TOPPADDING_RATIO).center().colspan(3);

            // RATING STARS
            getContentTable().row();
            Table ratingStarTable = new Table();
            ratingStarsEnabled = new Image[RATINGSTARS_COUNT];
            for (int i = 0; i < RATINGSTARS_COUNT; i++) {
                createRatingStar(i, ratingStarTable);
            }
            getContentTable().add(ratingStarTable).center();

            getContentTable().row();
            getContentTable().add(new Actor()).expand();
        }

        private void createRatingStar(int starNumber, Table ratingStartTable) {
            Stack ratingStack = new Stack();
            Image ratingStarDisabled = new Image(getTexture(VICTORYSCREEN_RATINGSTAR_DISABLED));
            Image ratingStarEnabled = new Image(getTexture(VICTORYSCREEN_RATINGSTAR_ENABLED));

            ratingStarEnabled.setVisible(false);
            ratingStarsEnabled[starNumber] = ratingStarEnabled;

            ratingStack.add(ratingStarDisabled);
            ratingStack.add(ratingStarEnabled);

            float ratingStarHeight = getWidth() * RATINGSTAR_HEIGHT_RATIO;
            float ratingStarSizeRatio = ratingStarHeight / ratingStarDisabled.getHeight();
            Cell<Stack> cell = ratingStartTable.add(ratingStack);
            cell.height(ratingStarHeight).width(ratingStarDisabled.getWidth() * ratingStarSizeRatio);
        }

        public Dialog show(int score, Stage stage) {
            if (score < 0 || score > RATINGSTARS_COUNT) {
                throw new IllegalArgumentException("Invalid score value: " + score);
            }

            super.show(stage);

            for (int i = 0; i < score; i++) {
                final Image ratingStarEnabled = ratingStarsEnabled[i];

                addAction(sequence(delay((i + 1) * RATINGSTAR_DELAY), run(new Runnable() {
                    @Override
                    public void run() {
                        ratingStarEnabled.setVisible(true);
                    }
                })));
            }
            return this;
        }

        @Override
        protected void leftButtonAction() {
        }

        @Override
        protected void rightButtonAction() {
            startNewGame();
        }
    }

    protected RandomAccess getRandomAccess() {
        return randomAccess;
    }

    protected GameDifficulty getGameDifficulty() {
        return gameDifficulty;
    }

    public void gameComplete(int score) {
        (new VictoryDialog()).show(score, getStage());
    }

    private void startNewGame() {
        performHideTransaction(new Runnable() {
            @Override
            public void run() {
                startLoading();
            }
        });
    }


//////////////////////////// HIDE/SHOW TRANSACTIONS

    protected void performHideTransaction(Runnable onHide) {
        if (!screenDimmer.isVisible()) {
            screenDimmer.addAction(alpha(0));
            screenDimmer.setVisible(true);
            screenDimmer.addAction(sequence(fadeIn(SCREEN_TRANSACTION_FADE_DELAY, Interpolation.pow4Out),
                                            run(onHide)));
        }
    }

    protected void performShowTransaction(Runnable onShow) {
        if (screenDimmer.isVisible()) {
            screenDimmer.addAction(sequence(
                    fadeOut(SCREEN_TRANSACTION_FADE_DELAY, Interpolation.pow4In),
                    run(new Runnable() {

                        @Override
                        public void run() {
                            screenDimmer.setVisible(false);
                        }
                    }),
                    run(onShow)));
        }
    }
}
