package cz.nic.tablexia.game.games.robbery;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Group;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.reflect.ClassReflection;
import com.badlogic.gdx.utils.reflect.ReflectionException;

import java.util.List;

import cz.nic.tablexia.game.games.robbery.assets.RobberyAssets;
import cz.nic.tablexia.game.games.robbery.creature.CreatureRoot;
import cz.nic.tablexia.screen.AbstractTablexiaScreen;
import cz.nic.tablexia.util.Log;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.alpha;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.fadeIn;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.fadeOut;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.moveTo;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.parallel;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.run;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.scaleTo;
import static com.badlogic.gdx.scenes.scene2d.actions.Actions.sequence;

public class GameScreen extends AbstractTablexiaScreen<Void> {

	private static class GameBackground extends Actor {
		
		private Texture texture;

		public GameBackground() {
			texture = RobberyScreen.getRobberyTexture(RobberyAssets.SCREEN_BACKGROUND_S5);
			texture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
			batch.disableBlending();
			batch.draw(texture, getX(), getY(), Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			batch.enableBlending();
		}
		
	}
	
	private static class FloorBackground extends Actor {
		
		private Texture texture;

		public FloorBackground() {
			texture = RobberyScreen.getRobberyTexture(RobberyAssets.SCREEN_NEWSSTAND_BOTTOM_S5);
			texture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
			batch.draw(texture, getX(), getY(), Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
		}
		
	}
	
	private static class GameForeground extends Actor {
		
		private Texture texture;

		public GameForeground() {
			texture = RobberyScreen.getRobberyTexture(RobberyAssets.SCREEN_NEWSSTAND_S5);
			texture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
			batch.draw(texture, getX(), getY(), Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
		}
		
	}
	
	public static class GenericInfoItem extends Actor {
		
		private Texture texture;

		public GenericInfoItem(String textureName) {
			texture = RobberyScreen.getRobberyTexture(textureName);
			texture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
			setSize(texture.getWidth(), texture.getHeight());
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
			Color color = getColor();
		    batch.setColor(color.r, color.g, color.b, color.a * parentAlpha);
			batch.draw(texture, getX(), getY(), getOriginX(), getOriginY(), getWidth(), getHeight(), getScaleX(), getScaleY(), getRotation(), 0, 0, texture.getWidth(), texture.getHeight(), false, false);
			batch.setColor(color.r, color.g, color.b, 1f);
		}
		
	}
	
	public static class ColorInfoBanner extends Actor {
		
		private Texture texture;

		public ColorInfoBanner(Color color) {
			setColor(color);
			texture = RobberyScreen.getRobberyTexture(RobberyAssets.CREATURE_BACKGROUND_TEXTURE);
			texture.setFilter(TextureFilter.Linear, TextureFilter.Linear);
			setSize(texture.getWidth(), texture.getHeight());
		}
		
		@Override
		public void draw(Batch batch, float parentAlpha) {
		    batch.setColor(getColor());
			batch.draw(texture, getX(), getY(), getOriginX(), getOriginY(), getWidth(), getHeight(), getScaleX(), getScaleY(), getRotation(), 0, 0, texture.getWidth(), texture.getHeight(), false, false);
			batch.setColor(Color.WHITE);
		}
		
	}
	
	private enum InfoItem {
		
		ARRESTED	(RobberyAssets.INFOITEM_ARRESTED,	COLOR_OK, 	RobberyAssets.SOUND_CHAINS),
		INNOCENCE	(RobberyAssets.INFOITEM_INNOCENCE, 	COLOR_KO, 	RobberyAssets.SOUND_ERROR),
		ALARM		(RobberyAssets.INFOITEM_ALARM, 		null, 		RobberyAssets.SOUND_ALARM);
		
		private final 	String 				textureName;
		private 		String 				soundName;
		private 		Color 				color;
		private 		ColorInfoBanner 	infoBanner;
		private 		GenericInfoItem 	infoItem;

		private InfoItem(String textureName, Color color, String soundName) {
			this.textureName = textureName;
			this.soundName = soundName;
			this.color = color;
		}
		
		private void init() {
			try {
				infoItem = (GenericInfoItem) ClassReflection.getConstructor(GenericInfoItem.class, String.class).newInstance(textureName);
				if (color != null) {
					infoBanner = (ColorInfoBanner) ClassReflection.getConstructor(ColorInfoBanner.class, Color.class).newInstance(color);
				}
			} catch (ReflectionException e) {
				Log.err(getClass(), "Cannot initialize info item!", e);
			}
		}
		
		public void show() {
			for (InfoItem container : InfoItem.values()) {
				container.infoItem.setVisible(false);
				if (container.infoBanner != null) {
					container.infoBanner.setVisible(false);
				}
			}
			if (infoBanner != null) {
				infoBanner.setVisible(true);
			}
			infoItem.addAction(alpha(0));
			infoItem.addAction(scaleTo(INFOITEM_INITIAL_SCALE, INFOITEM_INITIAL_SCALE));
			infoItem.setVisible(true);
			infoItem.addAction(parallel(fadeIn(INFOITEM_SHOW_DURATION, INFOITEM_SHOW_ALPHA_INTERPOLATION),
										scaleTo(1f, 1f, INFOITEM_SHOW_DURATION, INFOITEM_SHOW_SCALE_INTERPOLATION)));
			RobberyScreen.getRobberySound(soundName).play();
		}
		
		public void hide() {
			if (infoBanner != null && infoBanner.isVisible()) {
				infoBanner.setVisible(false);
			}
			if (infoItem.isVisible()) {			
				infoItem.addAction(sequence(parallel(fadeOut(INFOITEM_HIDE_DURATION, INFOITEM_HIDE_INTERPOLATION),
													 scaleTo(INFOITEM_INITIAL_SCALE, INFOITEM_INITIAL_SCALE, INFOITEM_HIDE_DURATION, INFOITEM_HIDE_INTERPOLATION)),
											run(new Runnable() {

												@Override
												public void run() {
													infoItem.setVisible(false);
												}
											})));
			}
		}
		
		public static void init(Group infoItemLayer, Group colorInfoLayer) {
			for (InfoItem container : InfoItem.values()) {
				container.init();
				container.infoItem.setVisible(false);
				infoItemLayer.addActor(container.infoItem);
				if (container.infoBanner != null) {
					container.infoBanner.setVisible(false);
					colorInfoLayer.addActor(container.infoBanner);
				}
			}
			calculateDimensions(infoItemLayer.getStage().getWidth(), infoItemLayer.getStage().getHeight());
		}
		
		public static void calculateDimensions(float width, float height) {
			float infoItemPositionX = width * INFOITEM_POSITION_X_RATIO;
			float infoItemPositionY = height * INFOITEM_POSITION_Y_RATIO;
			
			for (InfoItem container : InfoItem.values()) {
				float infoItemSizeRatio = (height * INFOITEM_SIZE_RATIO) / container.infoItem.getHeight();
				
				container.infoItem.setSize(container.infoItem.getWidth() * infoItemSizeRatio, container.infoItem.getHeight() * infoItemSizeRatio);
				container.infoItem.setPosition(infoItemPositionX - (container.infoItem.getWidth() / 2), infoItemPositionY);
				container.infoItem.setOrigin(Align.center);
				
				if (container.infoBanner != null) {					
					container.infoBanner.setSize(width * COLORBANNER_WIDTH_RATIO, height);
					container.infoBanner.setPosition(width * COLORBANNER_X_POSITION_RATIO, 0);
				}
			}
		}
		
	}
	
	private static final Color 			COLOR_OK	 						= Color.valueOf("39b54aff");
	private static final Color 			COLOR_KO	 						= Color.valueOf("c1272dff");
	private static final float 			COLORBANNER_X_POSITION_RATIO 		= 1f/2;
	private static final float 			COLORBANNER_WIDTH_RATIO 			= 1f/3;
	
	private static final float 			INFOITEM_POSITION_Y_RATIO 			= 3f/7;
	private static final float 			INFOITEM_POSITION_X_RATIO 			= 11f/36;
	private static final Interpolation 	INFOITEM_HIDE_INTERPOLATION 		= Interpolation.pow4In;
	private static final Interpolation 	INFOITEM_SHOW_ALPHA_INTERPOLATION 	= Interpolation.pow4Out;
	private static final Interpolation 	INFOITEM_SHOW_SCALE_INTERPOLATION 	= Interpolation.swingOut;
	private static final float 			INFOITEM_HIDE_DURATION 				= 0.1f;
	private static final float 			INFOITEM_SHOW_DURATION 				= 0.2f;
	private static final float 			INFOITEM_INITIAL_SCALE 				= 0.5f;
	
	private static final float 			CREATURE_SIZE_RATIO 				= 3f/5;
	private static final float 			INFOITEM_SIZE_RATIO 				= 2f/7;
	
    private static final float          CREATURE_STRAT_MOVE_DURATION 		= 1.5f;
    private static final float          CREATURE_MOVE_DURATION_STEP  		= 0.013f;
    private static final float          CREATURE_FROM_SCALE          		= 0.1f;
    private static final float          CREATURE_TO_SCALE            		= 1.05f;
    private static final float          CREATURE_FINAL_SCALE         		= 0.9f;
    
    private final static float 			CREATURE_MIDDLE_DELAY_MODIFIER 		= 0.5f;
	private final static float 			CREATURE_FINISH_DELAY_MODIFIER 		= 0.25f;
	
	private final static Interpolation 	CREATURE_START_INTERPOLATION 		= Interpolation.pow4Out;
	private final static Interpolation 	CREATURE_MIDDLE_INTERPOLATION 		= Interpolation.pow4In;
	private final static Interpolation 	CREATURE_FINISH_INTERPOLATION 		= Interpolation.linear;
	
	private final Group colorInfoLayer 	= new Group();
	private final Group creatureLayer 	= new Group();
	private final Group infoItemLayer 	= new Group();
	
	private float 	creatureStartPositionX;
	private float 	creatureStartPositionY;
	private float 	creatureMiddle1PositionX;
	private float 	creatureMiddle1PositionY;
	private float 	creatureMiddle2PositionX;
	private float 	creatureMiddle2PositionY;
	private float 	creatureFinishPositionX;
	private float 	creatureFinishPositionY;
	
	private int 	creatureNumber = -1;
	private boolean running;
	
	
//////////////////////////// SCREEN LIFECYCLE

	@Override
	public void screenLoaded() {
		running = false;
		
		getStage().addActor(new GameBackground());
		getStage().addActor(colorInfoLayer);
		getStage().addActor(new FloorBackground());
		getStage().addActor(creatureLayer);
		getStage().addActor(new GameForeground());
		getStage().addActor(infoItemLayer);
		
		InfoItem.init(infoItemLayer, colorInfoLayer);
		prepareCreaturePositions(getStage().getViewport().getWorldWidth(), getStage().getViewport().getWorldHeight());
	}
	
	@Override
	protected void screenVisible() {
		running = true;
		showNextCreature(RobberyScreen.getRobberyData().getCreatures());
	}
	
	@Override
	public void screenResized(int width, int height) {
		InfoItem.calculateDimensions(width, height);
		prepareCreaturePositions(width, height);
	}
	
	@Override
	protected void screenDisposed() {
		running = false;
	}
	
	
//////////////////////////// ROBBERY GAME
	
	private int getNumberOfMistakesInGame() {
		return 0;
	}
	
	private float getCreatureMoveDurationForCreatureNumber(int creatureNumber) {
        return CREATURE_STRAT_MOVE_DURATION - (CREATURE_MOVE_DURATION_STEP * creatureNumber);
    }
	
	private void prepareCreaturePositions(float viewportWidth, float viewportHeight) {
		creatureMiddle1PositionX = (viewportWidth / 5) * 3;
		creatureMiddle1PositionY = ((viewportHeight / 8));
        creatureStartPositionX 	= viewportWidth * (4f/5);
        creatureStartPositionY 	= creatureMiddle1PositionY + (((viewportHeight / 10) * 2) * 0.25f);
        creatureMiddle2PositionX = viewportWidth * (2f/5);
        creatureMiddle2PositionY = creatureStartPositionY;
        creatureFinishPositionX = 0;
        creatureFinishPositionY = creatureStartPositionY;
    }
	
	private void showNextCreature(final List<CreatureRoot> creatures) {
		creatureNumber++;
        if ((creatureNumber < creatures.size()) && (getNumberOfMistakesInGame() < 3)) {
        	final float duration = getCreatureMoveDurationForCreatureNumber(creatureNumber);
            final CreatureRoot creature = creatures.get(creatureNumber);
            float creatureRatio = (getStage().getHeight() * CREATURE_SIZE_RATIO) / creature.getHeight();
            creature.setSize(creature.getWidth() * creatureRatio, creature.getHeight() * creatureRatio);
            creature.setOrigin(Align.center);
            creature.setPosition(creatureStartPositionX, creatureStartPositionY);
            creature.setScale(CREATURE_FROM_SCALE);
            final InputListener inputListener = new InputListener() {
            	
            	@Override
            	public boolean touchDown(InputEvent event, float x, float y, int pointer, int button) {
            		creature.removeListener(this);	
            		if (creature.isThief()) {
            			creature.setRevealed(true);
                        InfoItem.ARRESTED.show();
//                        getGameManager().setExtraInt1AndSave(bankovniloupezActivity.getGameManager().getExtraInt1() + 1);
                    } else {
                    	InfoItem.INNOCENCE.show();
//                        getGameManager().setExtraInt3AndSave(bankovniloupezActivity.getGameManager().getExtraInt3() + 1);
                    }
            		
            		return true;
            	}
            	
            };
			creature.addListener(inputListener);
            creature.addAction(sequence(parallel(moveTo(creatureMiddle1PositionX, creatureMiddle1PositionY, duration, CREATURE_START_INTERPOLATION),
            									 scaleTo(CREATURE_TO_SCALE, CREATURE_TO_SCALE, duration, CREATURE_START_INTERPOLATION)),
            							parallel(moveTo(creatureMiddle2PositionX, creatureMiddle2PositionY, duration * CREATURE_MIDDLE_DELAY_MODIFIER, CREATURE_MIDDLE_INTERPOLATION),
            									 scaleTo(CREATURE_FINAL_SCALE, CREATURE_FINAL_SCALE, duration * CREATURE_MIDDLE_DELAY_MODIFIER, CREATURE_MIDDLE_INTERPOLATION),
            									 run(new Runnable() {
 													
 													@Override
 													public void run() {
 														InfoItem.ALARM.hide();
 													}
 												})),
            							parallel(moveTo(creatureFinishPositionX, creatureFinishPositionY, duration * CREATURE_FINISH_DELAY_MODIFIER, CREATURE_FINISH_INTERPOLATION),
            									 run(new Runnable() {
													
													@Override
													public void run() {
														//do not touch behind window
														creature.removeListener(inputListener);
														InfoItem.ARRESTED.hide();
														InfoItem.INNOCENCE.hide();
														if (creature.isThief() && !creature.isRevealed()) {
															InfoItem.ALARM.show();
//									                        getGameManager().setExtraInt2AndSave(getGameManager().getExtraInt2() + 1);
									                    }
														if (running) {															
															showNextCreature(creatures);
														}
													}							
												})),
            							run(new Runnable() {
            								
											@Override
											public void run() {
												creature.remove();
											}
										})));
            creatureLayer.addActor(creature);
            

//            IModifierListener<IEntity> firstAnimListener = new EntityModifierListenerAdapter() {
//                @Override
//                public void onModifierFinished(IModifier<IEntity> pModifier, IEntity pItem) {
//                    hideAlarmInfoItem();
//                }
//            };
//            IModifierListener<IEntity> secoundAnimListener = new EntityModifierListenerAdapter() {
//                @Override
//                public void onModifierFinished(IModifier<IEntity> pModifier, IEntity pItem) {
//                    if (creature.isThief() && !creature.isRevealed()) {
//                        showAlarm();
//                        getGameManager().setExtraInt2AndSave(getGameManager().getExtraInt2() + 1);
//                    }
//                    hideBackground();
//                    hideNotAlarmInfoItem();
//                    creatureNumber++;
//                    showNextCreature();
//
//                    scoreText.setText(String.valueOf(creatureNumber - getNumberOfMistakesInGame()));
//                }
//            };
//            IModifierListener<IEntity> thirdAnimListener = new EntityModifierListenerAdapter() {
//                @Override
//                public void onModifierFinished(IModifier<IEntity> pModifier, IEntity pItem) {
//                    detachEntityFromLayer(creaturesLayer, creature);
//                }
//            };
//
//            creature.enableMove(getCreatureMoveDurationForCreatureNumber(creatureNumber), creatureStartPositionX, creatureStartPositionY, creatureMiddlePositionX, creatureMiddlePositionY, creatureFinishPositionX, creatureFinishPositionY, CREATURE_FROM_SCALE, CREATURE_TO_SCALE, CREATURE_FINAL_SCALE, firstAnimListener, secoundAnimListener, thirdAnimListener);
//        } else {
//            scene.registerEntityModifier(new DelayModifier(VICTORY_SCREEN_DELAY) {
//
//                @Override
//                protected void onModifierFinished(IEntity pItem) {
//                    gameComplete();
//                }
//
//            });
        } else {
            RobberyScreen.gameRobberyComplete();
        }
    }
}
