package cz.nic.tablexia.menu;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.Texture.TextureFilter;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.math.Interpolation.PowOut;
import com.badlogic.gdx.scenes.scene2d.Action;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.Stack;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton.TextButtonStyle;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.Align;

import cz.nic.tablexia.TablexiaSettings;
import cz.nic.tablexia.bus.ApplicationBus;
import cz.nic.tablexia.bus.event.MainMenuPositionChangedEvent;
import cz.nic.tablexia.graphics.ColorTexture;
import cz.nic.tablexia.loader.application.ApplicationFontManager;
import cz.nic.tablexia.loader.application.ApplicationSoundManager;
import cz.nic.tablexia.loader.application.ApplicationTextureManager;

/**
 * MainMenu container
 *
 * @author Matyáš Latner
 */
public class MainMenu extends Stack {

    private static final float      MENU_MOVE_DURATION = 1f;
    private static final PowOut     MENU_MOVE_INTERPOLATION = Interpolation.pow4Out;
    private static final float      MENU_CONTROLLER_SIZE_RATIO = (1f / 7);
    private static final Color      MENU_TEXT_COLOR = Color.BLACK;
    private static final int        OPEN_MENU_BUTTON_WIDTH = 55;
    private static final int        OPEN_MENU_BUTTON_HEIGHT = 75;
    private static final int        OPEN_MENU_BUTTON_TOP_OFFSET = 60;

    private Table               menuTableLayout;
    private TextButtonStyle     menuButtonStyle;
    private Button              openCloseButton;
    private float               menuClosePositionX;
    private int                 menuOpenPositionX;
    private int                 menuPositionY;
    private Action              moveAction;
    private boolean             isMenuClose;
    private boolean             isMenuOpen;
    private boolean             isClosing;
    private boolean             isOpening;

    public MainMenu(float width, float height) {
        setBounds(0, 0, width, height);
        setDebug(TablexiaSettings.getInstance().isShowBoundingBoxes());
        Texture background = ApplicationTextureManager.getInstance().getTexture(ApplicationTextureManager.MAINMENU_BACKGROUND);
        background.setFilter(TextureFilter.Linear, TextureFilter.Linear);

        isClosing = false;
        isOpening = false;

        // menu layout
        menuTableLayout = new Table();
        menuTableLayout.setFillParent(true);
        menuTableLayout.setBackground(new TextureRegionDrawable(new TextureRegion(background)));
        addActor(menuTableLayout);

        // menu button style
        menuButtonStyle = new TextButtonStyle();
        menuButtonStyle.font = ApplicationFontManager.getInstance().getFont(ApplicationFontManager.FONT_ROBOTO_REGULAR);
        menuButtonStyle.fontColor = MENU_TEXT_COLOR;
        menuButtonStyle.up = new TextureRegionDrawable(new TextureRegion(new ColorTexture(50, 50, Color.BLUE)));

        // open close button
        openCloseButton = new Button(new TextureRegionDrawable(new TextureRegion(new ColorTexture(OPEN_MENU_BUTTON_WIDTH, OPEN_MENU_BUTTON_HEIGHT, Color.RED))));
        openCloseButton.addAction(Actions.alpha(0));
        openCloseButton.align(Align.topRight);
        openCloseButton.addListener(new ClickListener() {

            @Override
            public void clicked(InputEvent event, float x, float y) {
                toggleMenu();
            }
        });
        Table openCloseButtonContainer = new Table();
        openCloseButtonContainer.add(openCloseButton).expandX().padTop(OPEN_MENU_BUTTON_TOP_OFFSET).right();
        openCloseButtonContainer.row();
        openCloseButtonContainer.add().expandY();
        addActor(openCloseButtonContainer);

        prepareMenuPositions();
        closeMenu(false);
    }


//////////////////////////// MENU ITEMS

    public void initMenuItems() {
        menuTableLayout.reset();
        menuTableLayout.setDebug(TablexiaSettings.getInstance().isShowBoundingBoxes());

        for (final MainMenuDefinition menuItem : MainMenuDefinition.getItemsForMenu()) {
            addMenuItem(menuItem);
            IMenuItem[] subMenu = menuItem.getSubmenu();
            if (subMenu != null) {
                for (IMenuItem submenuItem : subMenu) {
                    addMenuItem(submenuItem);
                }
            }
        }
    }

    private void addMenuItem(final IMenuItem menuItem) {
        TextButton subMenuButton = new TextButton(menuItem.getTitle(), menuButtonStyle);
        subMenuButton.addListener(new ClickListener() {

            @Override
            public void clicked(InputEvent event, float x, float y) {
                menuItem.performAction();
                if (menuItem.isCloseMenu()) {
                    closeMenu(true);
                }
            }

        });
        menuTableLayout.add(subMenuButton).expandX();
        menuTableLayout.row();
    }


//////////////////////////// MENU MOVEMENT

    private void prepareMenuPositions() {
        isMenuOpen = false;
        isMenuClose = false;
        isOpening = false;
        isClosing = false;
        menuClosePositionX = -(getWidth() - (getWidth() * MENU_CONTROLLER_SIZE_RATIO));
        menuOpenPositionX = 0;
        menuPositionY = 0;
    }

    private void toggleMenu() {
        if (isOpening || isMenuOpen) {
            closeMenu(true);
        } else {
            openMenu(true);
        }
    }

    public void closeMenu(boolean animated) {
        if (!isMenuClose && !isClosing) {
            removeAction(moveAction);
            isMenuOpen = false;
            isOpening = false;
            isClosing = true;
            if (animated) {
                moveAction = Actions.sequence(
                        Actions.moveTo(menuClosePositionX, menuPositionY, MENU_MOVE_DURATION, MENU_MOVE_INTERPOLATION),
                        Actions.run(new Runnable() {

                            @Override
                            public void run() {
                                moveAction = null;
                                isClosing = false;
                                isMenuClose = true;
                                sendMenuPositionChangedEvent();
                            }
                        }));
                addAction(moveAction);
                ApplicationSoundManager.getInstance().getSound(ApplicationSoundManager.MAINMENU_CLOSE).play();
            } else {
                moveAction = null;
                isClosing = false;
                isMenuClose = true;
                setPosition(menuClosePositionX, menuPositionY);
                sendMenuPositionChangedEvent();
            }
        }
    }

    public void openMenu(boolean animated) {
        if (!isMenuOpen && !isOpening) {
            removeAction(moveAction);
            isMenuClose = false;
            isClosing = false;
            isOpening = true;
            sendMenuPositionChangedEvent();
            if (animated) {
                moveAction = Actions.sequence(
                        Actions.moveTo(menuOpenPositionX, menuPositionY, MENU_MOVE_DURATION, MENU_MOVE_INTERPOLATION),
                        Actions.run(new Runnable() {

                            @Override
                            public void run() {
                                moveAction = null;
                                isMenuOpen = true;
                                isOpening = false;
                            }
                        }));
                addAction(moveAction);
                ApplicationSoundManager.getInstance().getSound(ApplicationSoundManager.MAINMENU_OPEN).play();
            } else {
                moveAction = null;
                isMenuOpen = true;
                isOpening = false;
                setPosition(menuOpenPositionX, menuPositionY);
            }
        }
    }

    //////////////////////////// POSITION CHANGED EVENT

    private void sendMenuPositionChangedEvent() {
        ApplicationBus.getInstance().post(new MainMenuPositionChangedEvent(!isMenuClose)).asynchronously();
    }


//////////////////////////// MENU MOVE PROGRESS

    public float getMenuOpenPercentage() {
        return 1 - Math.abs(getX() / menuClosePositionX);
    }

}
